/* element.c - provide some basic information about a given chemical element */

struct element {
  char symbol[3];
  char name[20];
  float mass;
  char group[3];
} elements[] = {
  "Xx", "Xxxxxxxx", 9999.9, "X",
  "H", "Hydrogen", 1.0, "-",
  "He", "Helium", 4.0, "0",
  "Li", "Lithium", 6.9, "I",
  "Be", "Beryllium", 9.0, "II",
  "B", "Boron", 10.8, "III",
  "C", "Carbon", 12.0, "IV",
  "N", "Nitrogen", 14.0, "V",
  "O", "Oxygen", 16.0, "VI",
  "F", "Flourine", 19.0, "VII",
  "Ne", "Neon", 20.2, "0",
  "Na", "Sodium", 23.0, "I",
  "Mg", "Magnesium", 24.3, "II",
  "Al", "Aluminium", 27.0, "III",
  "Si", "Silicon", 28.1, "IV",
  "P", "Phosphorous", 31.0, "V",
  "S", "Sulphur", 32.1, "VI",
  "Cl", "Chlorine", 35.5, "VII",
  "Ar", "Argon", 39.9, "O",
  "K", "Potassium", 39.1, "I",
  "Ca", "Calcium", 40.1, "II",
  "Sc", "Scandium", 45.0, "",
  "Ti", "Titanium", 47.9, "",
  "V", "Vanadium", 50.9, "",
  "Cr", "Chromium", 52.0, "",
  "Mn", "Magnesium", 54.9, "",
  "Fe", "Iron", 55.8, "",
  "Co", "Cobalt", 58.9, "",
  "Ni", "Nickel", 58.7, "",
  "Cu", "Copper", 63.5, "",
  "Zn", "Zinc", 65.4, "",
  "Ga", "Gallenium", 69.7, "III",
  "Ge", "Germanium", 72.6, "IV",
  "As", "Arsenic", 74.9, "V",
  "Se", "Selenium", 79.0, "VI",
  "Br", "Bromine", 79.9, "VII",
  "Kr", "Krypton", 83.8, "0"
};

#define NUMELEMENTS (sizeof(elements) / sizeof(struct element))

struct shell {
  char name[3];
  int capacity;
  int electrons;
} shells[] = {
  "1s", 2, 0,
  "2s", 2, 0,
  "2p", 6, 0,
  "3s", 2, 0,
  "3p", 6, 0,
  "4s", 2, 0,
  "3d", 10, 0,
  "4p", 6, 0,
  "5s", 2, 0,
  "4d", 10, 0
};

#define NUMSHELLS (sizeof(shells) / sizeof(struct shell))

#define TRUE ((1==1))
#define FALSE ((0==1))

int code, mass, nucleons, protons, electrons, neutrons;
char massstring[20];
int isotopemode=FALSE, protonmode=FALSE;

main(int argc, char **argv)
{
  if ((argc < 2) 
      || (strcmp(argv[1],"--help") == 0)
      || (strcmp(argv[1],"-?") == 0)
      || (strcmp(argv[1],"-h") == 0)
      )
    {
      printf("Element by Adam Sampson\n",argv[0]);
      printf("Usage is %s <symbol | proton> [<mass>]\n",argv[0],argv[0]);
      exit(10);
    }
  code = findelement(argv[1]);
  if (code == 0)
    {
      printf("%s: Could not find specified element '%s'.\n", argv[0], argv[1]);
      printf("You need to enter the element's symbol with correct case,\n");
      printf("e.g. %s Xe\n", argv[0]);
      exit(10);
    }
  if (atoi(argv[2]) > 0) 
    {
      isotopemode = TRUE;
      mass = atoi(argv[2]);
      sprintf(massstring, "-%d", mass);
    }

  calculateparticles(code);
  if (protonmode == FALSE) elementinfo(code);
  electronconfig(electrons);
}

/* Return the code for a given element name or proton number. */
int findelement(char *symbol)
{
  int i, sym=0;
  for (i=1; i<NUMELEMENTS; i++) 
    if (strcmp(elements[i].symbol,symbol) == 0) sym = i;
  if ((atoi(symbol) > 0) && (atoi(symbol) < NUMELEMENTS))
    {
      sym=atoi(symbol); 
      protonmode=TRUE; 
    }
  return sym;
}

/* Calculate numbers of nucleons, protons, neutrons and electrons. */
int calculateparticles(int elcode)
{
  mass = elements[elcode].mass;
  sprintf(massstring, "");
  nucleons = mass;
  protons = elcode;
  neutrons = nucleons - protons;
  electrons = protons;  /* not an ion */
}

/* Display information about the element with proton number elcode. */
int elementinfo(int elcode)
{
  printf("%s%s (%s)\n", elements[elcode].name, massstring, 
	 elements[elcode].symbol);
  if (isotopemode == FALSE)
    printf("Mean RAM: %.1f", elements[elcode].mass);
  else
    printf("RAM: %d", mass);
  printf("  Proton number: %d", elcode);
  if (strlen(elements[elcode].group) > 0)
    printf("  Group: %s", elements[elcode].group);
  printf("\n%d nucleons: %d neutrons, %d protons; %d electrons\n",
	 nucleons, neutrons, protons, electrons);
}

/* Print an electron config diagram, in TeX format. */
int electronconfig(int etmp)
{
  int stmp;
  stmp = 0;
  while (etmp > 0)
    {
      if (shells[stmp].capacity < etmp)
	{
	  shells[stmp].electrons = shells[stmp].capacity;
	  etmp = etmp - shells[stmp].capacity;
	  stmp++;
	}
      else
	{
	  shells[stmp].electrons = etmp;
	  etmp = 0;
	}
    }
  stmp = 0;
  while (shells[stmp].electrons > 0)
    {
      printf("%s^{%d} ", shells[stmp].name, shells[stmp].electrons);
      stmp++;
    }
  printf("\n");
}








